"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generate = void 0;
const base_1 = require("@modular-service/base");
const path_1 = __importDefault(require("path"));
const postgresql_1 = require("@testcontainers/postgresql");
const kanel = __importStar(require("kanel"));
const kanel_zod_1 = require("kanel-zod");
const kanel_knex_1 = require("kanel-knex");
const recase_1 = require("@kristiandupont/recase");
const zod_1 = require("zod");
const Options = zod_1.z.object({
    zodTypeMap: zod_1.z.record(zod_1.z.string()).optional(),
    typeMap: zod_1.z.record(zod_1.z.string()).optional(),
    modelsType: zod_1.z.literal('knex').optional()
});
/**
 * Must be called AFTER declaring all migrations because only declared migrations will run to generate schema
 */
function generate(outputDirectory, options = {}) {
    const parsedOptions = Options.parse(options);
    const dirname = (0, base_1.getRuntimeDirname)();
    (0, base_1.deferGenerator)(async () => {
        const postgresContainer = await new postgresql_1.PostgreSqlContainer().start();
        try {
            // run migrations
            await base_1.Migration.migrate(dirname, {
                client: 'pg',
                connection: postgresContainer.getConnectionUri()
            });
            // generate schema
            const outputPath = path_1.default.join(dirname, outputDirectory);
            await kanel.processDatabase({
                connection: postgresContainer.getConnectionUri(),
                outputPath,
                ...kanelConfig(parsedOptions)
            });
        }
        finally {
            await postgresContainer.stop();
        }
        return {};
    });
}
exports.generate = generate;
function kanelConfig(options) {
    const toPascalCase = (0, recase_1.recase)('snake', 'pascal');
    const preRenderHooks = [
        (0, kanel_zod_1.makeGenerateZodSchemas)({
            getZodSchemaMetadata: kanel_zod_1.defaultGetZodSchemaMetadata,
            getZodIdentifierMetadata: kanel_zod_1.defaultGetZodIdentifierMetadata,
            zodTypeMap: {
                ...kanel_zod_1.defaultZodTypeMap,
                'pg_catalog.time': 'z.string()',
                'pg_catalog.uuid': 'z.string().uuid()',
                ...options.zodTypeMap
            }
        }),
        // generateMigrationCheck
    ];
    let typeFilter = undefined;
    if (options.modelsType === 'knex') {
        preRenderHooks.push(kanel_knex_1.generateKnexTablesModule);
        typeFilter = kanel_knex_1.knexTypeFilter;
    }
    return {
        preDeleteOutputFolder: true,
        resolveViews: true,
        preRenderHooks,
        typeFilter,
        // // Add a comment about the entity that the type represents above each type.
        // getMetadata: (details, generateFor) => {
        //   const { comment: strippedComment } = tryParse(details.comment);
        //   const isAgentNoun = ['initializer', 'mutator'].includes(generateFor ?? '');
        //   const relationComment = isAgentNoun
        //     ? `Represents the ${generateFor} for the ${details.kind} ${details.schemaName}.${details.name}`
        //     : `Represents the ${details.kind} ${details.schemaName}.${details.name}`;
        //   const suffix = isAgentNoun ? `_${generateFor}` : '';
        //   return {
        //     name: toPascalCase(details.name + suffix),
        //     comment: [relationComment, ...(strippedComment ? [strippedComment] : [])],
        //     path: path.join(outputPath, toPascalCase(details.name)),
        //   };
        // },
        // // Add a comment that says what the type of the column/attribute is in the database.
        // getPropertyMetadata: (property, _details, generateFor) => {
        //   const { comment: strippedComment } = tryParse(property.comment);
        //   return {
        //     name: property.name,
        //     comment: [
        //       `Database type: ${property.expandedType}`,
        //       ...(strippedComment ? [strippedComment] : []),
        //       ...(generateFor === 'initializer' && property.defaultValue
        //         ? [`Default value: ${property.defaultValue}`]
        //         : []),
        //     ]
        //   }
        // },
        // This implementation will generate flavored instead of branded types.
        // See: https://spin.atomicobject.com/2018/01/15/typescript-flexible-nominal-typing/
        generateIdentifierType: (c, d, config) => {
            const name = toPascalCase(d.name) + toPascalCase(c.name);
            const innerType = kanel.resolveType(c, d, {
                ...config,
                // Explicitly disable identifier resolution so we get the actual inner type here
                generateIdentifierType: undefined,
            });
            const imports = [];
            let type = innerType;
            if (typeof innerType === "object") {
                // Handle non-primitives
                type = innerType.name;
                imports.push(...innerType.typeImports);
            }
            return {
                declarationType: 'typeDeclaration',
                name,
                exportAs: 'named',
                typeDefinition: [`${type} & { __flavor?: '${name}' }`],
                typeImports: imports,
                comment: [`Identifier type for ${d.schemaName}.${d.name}`],
            };
        },
        customTypeMap: {
            // // The bytea package (https://www.npmjs.com/package/postgres-bytea) could be used for byte arrays.
            // 'pg_catalog.bytea': { name: 'bytea', typeImports: [{ name: 'bytea', path: 'bytea', isAbsolute: true, isDefault: true, importAsType: false }] },
            'pg_catalog.time': 'string',
            ...options.typeMap
        },
    };
}
//# sourceMappingURL=index.js.map